
BlacklistPolicy = class();

function BlacklistPolicy:Constructor(filterData)
	self.Name = "BlacklistPolicy";
	self.Data = filterData;
end

function BlacklistPolicy:Execute(message, channel, username, text, bag)
	self:CleanupBlacklist();

	if self.Data.BlackList[message] ~= nil then
		-- Message is on the black list
		return true;
	end
end

function BlacklistPolicy:Apply(message, channel, username, text, blocked, policyName, bag)
	if blocked == true then
		if self.Data.BlackList[message] then
			-- update
			self.Data.BlackList[message].Timestamp = Turbine.Engine:GetGameTime();

			Events.FireEvent(self, self.Updated, {});
		else
			-- add to black list
			self.Data.BlackList[message] = {
				["Timestamp"] = Turbine.Engine:GetGameTime(),
				["Origin"] = policyName,
			};

			--Turbine.Shell.WriteLine(text);
			--Turbine.Shell.WriteLine(string.gsub(utf8.printable(message), "[<>]", "#"));
			Events.FireEvent(self, self.Changed, {});
		end
	end
end

-- Cleanup the black list, remove outdated entries
function BlacklistPolicy:CleanupBlacklist()
	local count = 0;
	local modified = false;

	for key, entry in pairs(self.Data.BlackList) do
		if Turbine.Engine:GetGameTime() - entry.Timestamp > self.Data.BlackListTimeMax then
			self.Data.BlackList[key] = nil;
			modified = true;
		else
			count = count + 1;
		end
	end

	-- Seems to be very full, let's clear the black list
	if count > self.Data.BlackListThreshold then
		self.Data.BlackList = {};
		modified = true;
	end

	if modified then
		Events.FireEvent(self, self.Changed, {});
	end
end

-- removes all entries from the blacklist
function BlacklistPolicy:Clear()
	local modified = false;

	for key, _ in pairs(self.Data.BlackList) do
		self.Data.BlackList[key] = nil;
		modified = true;
	end
	
	if modified then
		Events.FireEvent(self, self.Changed, {});
	end
end

function BlacklistPolicy:Updated(args)
end

function BlacklistPolicy:Changed(args)
end
