
GoldsellerPolicy = class();

function GoldsellerPolicy:Constructor(filterData)
	self.Name = "GoldsellerPolicy";
	self.Data = filterData;

	Events.AddCallback(Settings, "Saving", function(sender, args)
		args["GoldsellerPolicy"] = self:ExportConfig(args);
	end);

	Events.AddCallback(Settings, "Loaded", function(sender, args)
		self:ImportConfig(args["GoldsellerPolicy"]);
	end);
end

function GoldsellerPolicy:Execute(message, channel, username, text, bag)
	local upperText = utf8.upper(text);
	local counter = 0;

	-- check for symbol
	for _, symbol in pairs(self.Data.SpamHeuristic.Symbols) do
		if string.find(upperText, symbol, 1, true) ~= nil then
			-- has a typical symbol used by gold sellers
			counter = counter + 1;
		end
	end

	-- check for word
	for word in utf8.words(upperText) do
		for _, w in pairs(self.Data.SpamHeuristic.Words) do
			if w == word then
				-- has a typical word used by gold sellers
				counter = counter + 1;
			end
		end
	end

	if not (counter < self.Data.SpamHeuristic.Threshold) then
		self:DebugOut("too many gold seller words or symbols detected (" .. counter .. ")");
		return true;
	end
end

function GoldsellerPolicy:Apply(message, channel, username, text, blocked, policyName, bag)
	-- not needed
end

function GoldsellerPolicy:DebugOut(message)
	if self.Data.Debug then
		Turbine.Shell.WriteLine(self.Name .. ":> " .. message);
	end
end

function GoldsellerPolicy:SetHeuristicTables(words, symbols)
	local trim = function(str)
		return string.match(str, "^%s*(.-)%s*$");
	end

	-- set
	local _words = {};

	if type(words) == "string" then
		for word in utf8.words(words) do
			local value = utf8.upper(utf8.printable(trim(word)));

			if utf8.len(value) > 0 then
				_words[value] = value;
			end
		end
	end

	if type(words) == "table" then
		for _, word in pairs(words) do
			local value = utf8.upper(utf8.printable(trim(word)));

			if utf8.len(value) > 0 then
				_words[value] = value;
			end
		end
	end
	
	-- set
	local _symbols = {};

	if type(symbols) == "string" then
		for symbol in string.gmatch(symbols, "[^ ]+") do
			local value = utf8.upper(utf8.printable(trim(symbol)));

			if utf8.len(value) > 0 then
				_symbols[value] = value;
			end
		end
	end

	if type(symbols) == "table" then
		for _, symbol in pairs(symbols) do
			local value = utf8.upper(utf8.printable(trim(symbol)));

			if utf8.len(value) > 0 then
				_symbols[value] = value;
			end
		end
	end
	
	self.Data.SpamHeuristic.Words = {};
	
	for k, v in pairs(_words) do
		table.insert(self.Data.SpamHeuristic.Words, k);
	end
	
	self.Data.SpamHeuristic.Symbols = {};

	for k, v in pairs(_symbols) do
		table.insert(self.Data.SpamHeuristic.Symbols, k);
	end

	Events.FireEvent(self, self.HeuristicTablesChanged, {});
end

function GoldsellerPolicy:HeuristicTablesChanged(args)
end

function GoldsellerPolicy:SetThreshold(threshold)
	if threshold == self.Data.SpamHeuristic.Threshold then
		return;
	end

	self.Data.SpamHeuristic.Threshold = threshold;
	Events.FireEvent(self, self.ThresholdChanged, {});
end

function GoldsellerPolicy:ThresholdChanged(args)
end

function GoldsellerPolicy:ExportConfig()
	local config = {
		["HeuristicWords"]     = self.Data.SpamHeuristic.Words,
		["HeuristicSymbols"]   = self.Data.SpamHeuristic.Symbols,
		["HeuristicThreshold"] = self.Data.SpamHeuristic.Threshold,
	};

	return config;	
end

function GoldsellerPolicy:ImportConfig(config)
	if config == nil then
		return;
	end

	local function array(tbl)
		local arr = {};

		for k, v in pairs(tbl) do
			table.insert(arr, v);
		end

		return arr;
	end

	if type(config.HeuristicWords) == "table" then
		self.Data.SpamHeuristic.Words = array(config.HeuristicWords);
	end

	if type(config.HeuristicSymbols) == "table" then
		self.Data.SpamHeuristic.Symbols = array(config.HeuristicSymbols);
	end

	if type(config.HeuristicThreshold) == "number" then
		self:SetThreshold(config.HeuristicThreshold);
	end

	Events.FireEvent(self, self.HeuristicTablesChanged, {});
end
