
QualityPolicy = class();

function QualityPolicy:Constructor(filterData)
	self.Name = "QualityPolicy";
	self.Data = filterData;
end

function QualityPolicy:Execute(message, channel, username, text, bag)
	local upperText = utf8.upper(text);
	local textLength = utf8.len(string.gsub(upperText, "%s+", "")); -- without whitespace gives better results
	local wordsLength = 0;
	local trashCount = 0;
	local uniqueWords = {};

	for word in utf8.words(upperText) do
		if utf8.len(word) > 2 then
			uniqueWords[word] = (uniqueWords[word] or 0) + 1;
		else
			trashCount = trashCount + 1;
		end
	end

	if trashCount > self.Data.ChatQualityMaxTrash then
		-- i d o n t k n o w m a y b e s p a m
		self:DebugOut("to many trash (" .. trashCount .. ")");
		return true;
	end

	for word in pairs(uniqueWords) do
		wordsLength = wordsLength + utf8.len(word);
	end

	if textLength > 0 and wordsLength > 0 then
		local messageQuality = wordsLength / textLength;
		local lowerLimit = self.Data.ChatQuality * self.Data.ChatQualityThreshold;

		bag[self.Name] = {["MessageQuality"] = messageQuality, ["MessageLength"] = textLength};

		if messageQuality < lowerLimit then
			if not (textLength > 1.5 * self.Data.ChatMessageLength) then
				-- for usability don't block on normal or short messages
				return;
			end

			-- maybe spam
			self:DebugOut("bad quality: chat " .. string.format("%.2f", self.Data.ChatQuality) .. " lower limit " .. string.format("%.2f", lowerLimit) .. " > message " .. string.format("%.2f", messageQuality));
			return true;
		end	
	end
end

function QualityPolicy:Apply(message, channel, username, text, blocked, policyName, bag)
	if blocked then
		return;
	end

	if not bag[self.Name] then
		return;
	end

	-- moving average
	self.Data.ChatQualityCounter = self.Data.ChatQualityCounter + 1;
	self.Data.ChatQuality = self.Data.ChatQuality + (bag[self.Name].MessageQuality - self.Data.ChatQuality) / self.Data.ChatQualityCounter;

	-- moving average
	self.Data.ChatMessageCounter = self.Data.ChatMessageCounter + 1;
	self.Data.ChatMessageLength = self.Data.ChatMessageLength + (bag[self.Name].MessageLength - self.Data.ChatMessageLength) / self.Data.ChatMessageCounter;
end

function QualityPolicy:DebugOut(message)
	if self.Data.Debug then
		Turbine.Shell.WriteLine(self.Name .. ":> " .. message);
	end
end
