--[[

Naive Bayes Classifier
https://hackernoon.com/how-to-build-a-simple-spam-detecting-machine-learning-classifier-4471fe6b816e

--]]

--[[

TODO (or not todo)

tf-idf : term frequency-inverse document frequency

tf-idf = tf(w) * idf(w)

tf  = (number of times the word appears in a document) / (total number of words in the document)
idf = log(number of documents / number of documents that contain word)

--]]

SpamPolicy = class();

function SpamPolicy:Constructor(filterData)
	self.Name = "SpamPolicy";
	self.Data = filterData;

	self._isSpamWords     = {};
	self._numIsSpamWords  = 0;

	self._noSpamWords     = {};
	self._numNoSpamWords  = 0;

	Events.AddCallback(Settings, "Saving", function(sender, args)
		args["SpamPolicy"] = self:ExportConfig(args);
	end);

	Events.AddCallback(Settings, "Loaded", function(sender, args)
		self:ImportConfig(args["SpamPolicy"]);
	end);
end

function SpamPolicy:Execute(message, channel, username, text, bag)
	local upperText = utf8.upper(text);
	local words = self:GetWords(upperText);

	local countIsSpam = 0;
	local countNoSpam = 0;

	for _, word in pairs(words) do
		countIsSpam = countIsSpam + (self._isSpamWords[word] or 0);
		countNoSpam = countNoSpam + (self._noSpamWords[word] or 0);
	end

	local tfIsSpam = countIsSpam / math.max(1, self._numIsSpamWords);
	local tfNoSpam = countNoSpam / math.max(1, self._numNoSpamWords);

	--
	local total = self._numIsSpamWords + self._numNoSpamWords;

	local pA    = self._numIsSpamWords / math.max(1, total);
	local pNotA = self._numNoSpamWords / math.max(1, total);

	local isSpam    = pA    * tfIsSpam;
	local isNotSpam = pNotA * tfNoSpam;
	--

	-- 
	local blocked = isSpam > isNotSpam;

	if blocked == true then
		if not (isNotSpam > 0) then
			-- can it be, that the spam has words who nobody used before?
			blocked = false;
		else
			self:DebugOut("is spam " .. isSpam .. " > is not spam " .. isNotSpam .. " = " .. tostring(blocked));
		end
	end

	return blocked;
end

function SpamPolicy:Apply(message, channel, username, text, blocked, policyName, bag)
	-- learn from a small set of policies or from successful posts
	if 	policyName == "GoldsellerPolicy" or
		policyName == "QualityPolicy" or
		policyName == nil then
		self:Train(text, blocked);
		return;
	end
end

function SpamPolicy:Train(text, blocked)
	local upperText = utf8.upper(text);
	local words = self:GetWords(upperText);

	if blocked == true then
		for _, word in pairs(words) do
			self._numIsSpamWords    = (self._numIsSpamWords    or 0) + 1;
			self._isSpamWords[word] = (self._isSpamWords[word] or 0) + 1;
		end
	end

	if blocked == false then
		for _, word in pairs(words) do
			self._numNoSpamWords    = (self._numNoSpamWords    or 0) + 1;
			self._noSpamWords[word] = (self._noSpamWords[word] or 0) + 1;
		end
	end
end

function SpamPolicy:GetWords(text)
	local words = {};

	for word in utf8.words(text) do
		if utf8.len(word) > 2 then
			table.insert(words, word);
		end
	end

	return words;
end

function SpamPolicy:DebugOut(message)
	if self.Data.Debug then
		Turbine.Shell.WriteLine(self.Name .. ":> " .. message);
	end
end

function SpamPolicy:ResetIsSpam()
	self._isSpamWords     = {};
	self._numIsSpamWords  = 0;

	self:DebugOut("policy was reseted");
	Events.FireEvent(self, self.Reseted, {});
end

function SpamPolicy:ResetNoSpam()
	self._noSpamWords     = {};
	self._numNoSpamWords  = 0;

	self:DebugOut("policy was reseted");
	Events.FireEvent(self, self.Reseted, {});
end

function SpamPolicy:Reseted(args)
end

function SpamPolicy:ExportConfig()
	local config = {
		["_isSpamWords"]     = self._isSpamWords,
		["_numIsSpamWords"]  = self._numIsSpamWords,

		["_noSpamWords"]     = self._noSpamWords,		
		["_numNoSpamWords"]  = self._numNoSpamWords,
	};

	return config;	
end

function SpamPolicy:ImportConfig(config)
	if config == nil then
		return;
	end
	
	if type(config._isSpamWords) == "table" then
		self._isSpamWords = config._isSpamWords;
	end
	
	if type(config._numIsSpamWords) == "number" then
		self._numIsSpamWords = config._numIsSpamWords;
	end
	
	if type(config._noSpamWords) == "table" then
		self._noSpamWords = config._noSpamWords;
	end

	if type(config._numNoSpamWords) == "number" then
		self._numNoSpamWords = config._numNoSpamWords;
	end
end
