
BaseWindow = class(Turbine.UI.Lotro.Window);

function BaseWindow:Constructor()
	Turbine.UI.Lotro.Window.Constructor(self);

	self.ContentLeft   = 20;
	self.ContentTop    = 50;
	self.ContentRight  = 20;
	self.ContentBottom = 40;
	
	self.Content = ContentControl();
	self.Content:SetParent(self);
	self.Content:SetPosition(self.ContentLeft, self.ContentTop);

	self:SetWantsUpdates(false);
	self:SetResizable(true);
end

function BaseWindow:SetContent(control)
	self.Content:SetContent(control);
	return self.Content;
end

function BaseWindow:SizeChanged(args)
	-- the user is sizing the window
	self:WindowLayout();
end

function BaseWindow:WindowLayout()
	-- block layout calls
	self._windowLayout = 1;

	local left = self:GetLeft(self);
	local top = self:GetTop(self);

	self.Content:SetSize(self:GetClientSize());

	local contentWidth, contentHeight = self.Content:GetSize();
	local clientWidth, clientHeight = self:GetClientSize();

	if contentWidth > clientWidth then
		self:SetMinimumWidth(contentWidth + self.ContentLeft + self.ContentRight);
		self:SetPosition(left, top);
		-- don't reset _windowLayout
		return;
	end

	if contentHeight > clientHeight then
		self:SetMinimumHeight(contentHeight + self.ContentTop  + self.ContentBottom);
		self:SetPosition(left, top);
		-- don't reset _windowLayout
		return;
	end

	self._windowLayout = 0;
	self:Layout();
end

function BaseWindow:Layout()
	if self._windowLayout == 1 then
		-- window layout in progress, don't disturb
		return;
	end

	self.Content:SetSize(self:GetClientSize());
	self.Content:PerformLayout();
	self:SetClientSize(self.Content:GetSize());
end

function BaseWindow:GetClientSize()
	local width  = self:GetWidth()  - (self.ContentLeft + self.ContentRight);
	local height = self:GetHeight() - (self.ContentTop  + self.ContentBottom);
	return width, height;
end

function BaseWindow:SetClientSize(width, height)
	local windowWidth  = math.max(0, width)  + self.ContentLeft + self.ContentRight;
	local windowHeight = math.max(0, height) + self.ContentTop  + self.ContentBottom;
	self:SetSize(windowWidth, windowHeight);
end

function BaseWindow:FitToContent()
	self:Layout();
	self:SetMinimumSize(self:GetSize());
end

function BaseWindow:Invalidate()
	self.Content:Invalidate();
end

function BaseWindow:CenterWindow()
	local left = Turbine.UI.Display:GetWidth() / 2 - self:GetWidth() / 2;
	local top = Turbine.UI.Display:GetHeight() / 2 - self:GetHeight() / 2;
	self:SetPosition(left, top);
end

function BaseWindow:Closing(args)
	self:SetWantsUpdates(false);
	self.Content:Destroy();

	for k, _ in pairs(self.Content) do
		self.Content[k] = nil;
	end

	self.Content = nil;
	
	for k, _ in pairs(self) do
		self[k] = nil;
	end
end
