
BlacklistPolicyWindow = class(BaseWindow);

function BlacklistPolicyWindow:Constructor(filterEngine)
	BaseWindow.Constructor(self);
	self.Engine = filterEngine;
	self.Policy = self.Engine:GetPolicy("BlacklistPolicy");
	self._autoRefresh = true;

	local bindings = {
		["RefreshButtonClick"] = function(sender, args)
			self:UpdateStats();
		end,

		["ClearButtonClick"] = function(sender, args)
			self.Policy:Clear();
			self:UpdateStats();
		end,

		["AutoRefreshCheckBoxName"] = "AutoRefreshCheckBox",
		["AutoRefreshChecked"] = self._autoRefresh,
		["AutoRefreshCheckedChanged"] = function(sender, args)
			self._autoRefresh = sender:IsChecked();
			self:UpdateStats();
		end,
	};

	local content = TemplateLoader.Setup(self, self:GetTemplate(bindings));
	self:SetContent(content);
	self:RegisterCallbacks();
	self:SetText("Blacklist Policy");
	self:FitToContent();
	self:SetMaximumSize(1000, 790);
	self:UpdateStats();
end

function BlacklistPolicyWindow:RegisterCallbacks()
	self._registrations = self._registrations or {};
	
	-- helper
	local function add(object, event, callback)
		Events.AddCallback(object, event, callback);
		table.insert(self._registrations, {["object"] = object, ["event"] = event, ["callback"] = callback});
	end

	add(self.Policy, "Changed", function(sender, args)
		self:UpdateStats(args);
	end);
end

function BlacklistPolicyWindow:UnregisterCallbacks()
	if self._registrations then
		for _, reg in pairs(self._registrations) do
			Events.RemoveCallback(reg.object, reg.event, reg.callback);
		end

		self._registrations = nil;
	end
end

function BlacklistPolicyWindow:Closing(args)
	self:UnregisterCallbacks();
	self.Engine = nil;
	self.Policy = nil;
	BaseWindow.Closing(self, args);
end

function BlacklistPolicyWindow:UpdateStats(args)
	if self._autoRefresh == false then
		return;
	end
	
	self.ItemContainer:Clear();

	for key, entry in pairs(self.Engine.Data.BlackList) do
		local bindings = {["Text"] = key, ["Origin"] = "Origin: <rgb=#b9b700>" .. entry.Origin .."</rgb>"};
		local control = TemplateLoader.Setup(self, self:GetItemTemplate(bindings));
		self.ItemContainer:AddControl(control);
	end

	self:Invalidate();
end

function BlacklistPolicyWindow:GetTemplate(bindings)
	return {
		["Type"] = StackPanel,
		["Width"] = 400,
		["Height"] = 300,
		["FitToContent"] = false,
		["Content"] =
		{
			{
				["Type"] = Border,
				["BorderWidth"] = 1,
				["BorderColor"] = Turbine.UI.Color(0.98, 0.4, 0.2, 0.2),
				["Content.DynamicHeight"] = 1,
				["Content"] = 
				{
					{
						["Type"] = Image,
						["ImageMode"] = ImageMode.KeepImage,
						["Background"] = Images.Blacklist,
						["BlendMode"] = Turbine.UI.BlendMode.Multiply,
						["ImageBackColor"] = Turbine.UI.Color(0.98, 0.2, 0.1, 0.1),
						["BackColor"] = Turbine.UI.Color(0.98, 0.2, 0.1, 0.1),
						["Content"] =
						{
							{
								["Type"] = ScrollPanel,
								["Content"] = 
								{
									{
										["Type"] = StackPanel,
										["Name"] = "ItemContainer",
										["Content.Padding"] = 4,
									},
								},
							},
						},
					},
				},
			},
			{
				["Type"] = StackPanel,
				["IsVertical"] = false,
				["Content.PaddingTop"] = 10,
				["Content.DynamicWidth"] = 1,
				["Content"] = 
				{
					{
						["Type"] = Button,
						["Text"] = "Clear",
						["Content.MaxWidth"] = 150,
						["Content.PaddingTop"] = 10,
						["Content.PaddingRight"] = 4,
						["Content.PaddingBottom"] = 4,
						["Content.HorizontalAlignment"] = HorizontalAlignment.Center,
						["Content.DynamicWidth"] = 1,
						["Click"] = bindings["ClearButtonClick"],
					},
					{
						["Type"] = Button,
						["Text"] = "Refresh",
						["Content.MaxWidth"] = 150,
						["Content.PaddingTop"] = 10,
						["Content.PaddingRight"] = 4,
						["Content.PaddingBottom"] = 4,
						["Content.HorizontalAlignment"] = HorizontalAlignment.Center,
						["Content.DynamicWidth"] = 1,
						["Click"] = bindings["RefreshButtonClick"],
					},
				},
			},
			{
				["Type"] = Border,
				["BorderWidth"] = {["Top"] = 1},
				["Content.PaddingTop"] = 10,
				["Content.PaddingBottom"] = 10,
			},
			{
				["Type"] = CheckBox,
				["Name"] = bindings["AutoRefreshCheckBoxName"],
				["Text"] = "Auto Refresh",
				["Checked"] = bindings["AutoRefreshChecked"],
				["CheckedChanged"] = bindings["AutoRefreshCheckedChanged"],
			},
		},
	};
end

function BlacklistPolicyWindow:GetItemTemplate(bindings)
	return {
		["Type"] = StackPanel,
		["Content.Padding"] = 4,
		["Content"] =
		{
			{
				["Type"] = TextBlock,
				["Text"] = bindings["Text"],
				["Content.PaddingTop"] = 10,
			},
			{
				["Type"] = TextBlock,
				["Text"] = bindings["Origin"],
				["Content.PaddingTop"] = 10,
			},
			{
				["Type"] = Border,
				["BorderWidth"] = {["Bottom"] = 1},
				["BorderColor"] = Turbine.UI.Color(0.98, 0.3, 0.2, 0.2),
			},
		},
	};
end