

_G.Button = class(LayoutBase);

_G.ButtonStyle = 
{
	["BorderColor"] = Turbine.UI.Color(0.9, 0.4, 0.4, 0.4),
	["BorderWidth"] = 3,

	["Font"] = Turbine.UI.Lotro.Font.Verdana14,
	["ForeColor"] = Turbine.UI.Color(0.7, 0.7, 0.5),
	["FontStyle"] = Turbine.UI.FontStyle.None,
	["OutlineColor"] = Turbine.UI.Color(0, 0, 0),
	["Padding"] = 4,

	["Image"] = nil,	
	["ImageMode"] = ImageMode.KeepImage,
	["ImageHorizontalAlignment"] = HorizontalAlignment.Center,
	["ImageVerticalAlignment"] = VerticalAlignment.Center,
	["ImageBlendMode"] = Turbine.UI.BlendMode.Multiply,

	["MouseDownColor"] = Turbine.UI.Color(0.9, 0.1, 0.1, 0.2),
	["MouseEnterColor"] = Turbine.UI.Color(0.9, 0.1, 0.2, 0.4),
	["MouseLeaveColor"] = Turbine.UI.Color(0.9, 0.1, 0.1, 0.3),
};

function Button:Constructor()
	LayoutBase.Constructor(self);

	self._mouseEnterColor = ButtonStyle.MouseEnterColor;
	self._mouseLeaveColor = ButtonStyle.MouseLeaveColor;
	self._mouseDownColor  = ButtonStyle.MouseDownColor;

	-- content
	local content = TemplateLoader.Setup(self, self:GetTemplate());
	content:SetParent(self);
	self.Content = content;

	-- self
	self:SetSize(self.Content:GetSize());
end

function Button:SetContent(control)
	return self.Image:SetContent(control);
end

function Button:SetText(text)
	self.TextBlock:SetText(text);
end

function Button:Layout()
	self.Content:SetSize(self:GetSize());
	self.Content:PerformLayout();
	self:SetSize(self.Content:GetSize());
	--self:PerformLayout();
end

function Button:SetBackground(image)
	self.Border:SetBackground(image);
end

function Button:SetBorderWidth(value)
	self.Border:SetBorderWidth(value);
end

function Button:SetBorderColor(color)
	self.Border:SetBorderColor(color);
end

function Button:SetMouseEnterColor(color)
	self._mouseEnterColor = color;
	self.Border:SetBackColor(self._mouseEnterColor);
end

function Button:SetMouseLeaveColor(color)
	self._mouseLeaveColor = color;
	self.Border:SetBackColor(self._mouseLeaveColor);
end

function Button:SetMouseDownColor(color)
	self._mouseDownColor = color;
	self.Border:SetBackColor(self._mouseDownColor);
end

function Button:MouseClick(args)
	Events.FireEvent(self, self.Click, args);
end

function Button:MouseEnter(args)
	self.Border:SetBackColor(self._mouseEnterColor);
end

function Button:MouseLeave(args)
	self.Border:SetBackColor(self._mouseLeaveColor);
end

function Button:MouseDown(args)
	self.Border:SetBackColor(self._mouseDownColor);
end

function Button:MouseUp(args)
	local x, y = self:GetMousePosition();
	local w, h = self:GetSize();

	-- is in client rect
	if x < 0 or x > w or y < 0 or y > h then
		self.Border:SetBackColor(self._mouseLeaveColor);
		return;
	end

	self.Border:SetBackColor(self._mouseEnterColor);
end

function Button:GetTemplate(bindings)
	return {
		["Type"] = Border,
		["Width"] = 100,
		["Name"] = "Border",
		["BackColor"] = ButtonStyle.MouseLeaveColor,
		["BorderColor"] = ButtonStyle.BorderColor,
		["BorderWidth"] = ButtonStyle.BorderWidth,
		["Content"] = 
		{
			{
				["Type"] = Image,
				["Name"] = "Image",
				["Height"] = 0,
				["ImageMode"] = ButtonStyle.ImageMode,
				["Background"] = ButtonStyle.Image,
				["BlendMode"] = ButtonStyle.ImageBlendMode,
				["HorizontalAlignment"] = ButtonStyle.ImageHorizontalAlignment,
				["VerticalAlignment"] = ButtonStyle.ImageVerticalAlignment,
				["Content.DynamicWidth"] = 1,
				["Content"] =
				{
					{
						["Type"] = TextBlock,
						["Name"] = "TextBlock",
						["TextAlignment"] = Turbine.UI.ContentAlignment.MiddleCenter,
						["Text"] = "",
						["Height"] = 20,
						["Font"] = ButtonStyle.Font,
						["FontStyle"] = ButtonStyle.FontStyle,
						["ForeColor"] = ButtonStyle.ForeColor,
						["OutlineColor"] = ButtonStyle.OutlineColor,
						["Content.Padding"] = ButtonStyle.Padding,
						["Content.DynamicWidth"] = 1,
					},
				},
			},
		},
	};
end
