
_G.FlowPanel = class(LayoutBase);

function FlowPanel:Constructor()
	LayoutBase.Constructor(self);

	self.ChildControls = {};
	self.FitToContent = true;
	self:SetSize(0, 0);
	self:SetWantsUpdates(false);
end

function FlowPanel:AddControl(control)
	local content = ContentControl();
	content:SetParent(self);
	content:SetContent(control);
	table.insert(self.ChildControls, content);
	return content;
end

function FlowPanel:SetFitToContent(value)
	self.FitToContent = (value == true);
end

function FlowPanel:Layout()
	-- need a width to flow
	local width = self:GetWidth();
	local height = self:GetHeight();

	if self.FitToContent == true then
		height = 0;
	end

	local leftPosition = 0;
	local topPosition = 0;

	--local lineHeight = 0;
	local line = {["height"] = 0, ["width"] = 0, ["sumDynamicWidth"] = 0, ["items"] = {}};
	local rows = {};
	local numRows = 0;

	-- build all rows
	for _, child in pairs(self.ChildControls) do
		local childWidth = math.min(width, child:GetWidth());
		local childMinWidth = child:GetMinWidth();
		local childMaxWidth = child:GetMaxWidth();

		if child.ContentStyle.Size.DynamicWidth > 0 then
			childWidth = math.min(width, child:GetWidth());
		end

		if childMinWidth > 0 and childWidth < childMinWidth then
			childWidth = childMinWidth;
		end

		if childMaxWidth > 0 then
			if childWidth > childMaxWidth then
				childWidth = childMaxWidth;
			end
			
			if childMaxWidth < width then
				childWidth = childMaxWidth;
			end

			if width > childMinWidth and width < childMaxWidth then
				childWidth = width;
			end
		end

		if not (child.ContentStyle.Size.DynamicWidth > 0) then
			child:SetWidth(childWidth);
			child:PerformLayout();
			childWidth = child:GetWidth();
		end

		local nextLeftPosition = leftPosition + childWidth;

		if nextLeftPosition > width and leftPosition > 0 then
			-- new line
			leftPosition = 0;
			numRows = numRows + 1;
			rows[numRows] = line;
			line = {["height"] = 0, ["width"] = 0, ["sumDynamicWidth"] = 0, ["items"] = {}};
		end

		table.insert(line.items, {["child"] = child, ["width"] = childWidth});
		line.width = line.width + childWidth;
		line.sumDynamicWidth = line.sumDynamicWidth + math.max(0, child.ContentStyle.Size.DynamicWidth);

		leftPosition = leftPosition + childWidth;
		width = math.max(width, leftPosition);
	end

	-- don't forget	the last line
	numRows = numRows + 1;
	rows[numRows] = line;

	-- horizontal position and widths
	for i = 1, numRows do
		leftPosition = 0;
		line = rows[i];

		local availableSpace = width - line.width;

		for _, item in pairs(line.items) do
			item.child:SetPosition(leftPosition, topPosition);

			if item.child.ContentStyle.Size.DynamicWidth > 0 then
				local factor = item.child.ContentStyle.Size.DynamicWidth / line.sumDynamicWidth;
				local space = math.floor(factor * availableSpace);

				item.child:SetWidth(item.width + space);
				item.child:PerformLayout();

				line.sumDynamicWidth = line.sumDynamicWidth - item.child.ContentStyle.Size.DynamicWidth;
				availableSpace = availableSpace + item.width - item.child:GetWidth();
			end

			line.height = math.max(line.height, item.child:GetHeight());
			leftPosition = leftPosition + item.child:GetWidth();
		end

		topPosition = topPosition + line.height;
		height = math.max(height, topPosition);
	end

	self:SetSize(width, height);
	--self:PerformLayout();
end

