

_G.Image = class(LayoutBase);

_G.ImageMode = {
	["KeepImage"] = 0,
	["RepeatImage"] = 1,
	["FitToImage"] = 2,
	["StretchImage"] = 3,
};

function Image:Constructor()
	LayoutBase.Constructor(self);

	-- background
	self.Background = Turbine.UI.Control();
	self.Background:SetParent(self);

	self.BackgroundImage = Turbine.UI.Control();
	self.BackgroundImage:SetParent(self.Background);
	
	-- content
	self.Content = ContentControl();
	self.Content:SetParent(self);

	-- self
	self.ImageMode = ImageMode.FitToImage;
	self.VerticalAlignment = VerticalAlignment.None;
	self.HorizontalAlignment = HorizontalAlignment.None;
end

function Image:SetContent(control)
	self.Content:SetContent(control);
	return self.Content;
end

function Image:Destroy()
	self.BackgroundImage:SetParent(nil);
	self.BackgroundImage = nil;

	self.Background:SetLeft(-Turbine.UI.Display:GetWidth());
	self.Background:SetParent(nil);
	self.Background = nil;

	LayoutBase.Destroy(self);
end

function Image:SetBackground(image)
	self.BackgroundImage:SetSize(0, 0);
	self.BackgroundImage:SetBackground(image);
	self.BackgroundImage:SetStretchMode(2);
	self.BackgroundImage:SetStretchMode(0);
	self.Background:SetSize(self.BackgroundImage:GetSize());
end

function Image:SetImageBackColor(color)
	self.Background:SetBackColor(color);
end

function Image:SetBlendMode(blendMode)
	self.BackgroundImage:SetBlendMode(blendMode);
end

function Image:SetImageMode(mode)
	self.ImageMode = mode;
end

function Image:SetVerticalAlignment(alignment)
	self.VerticalAlignment = alignment;
end

function Image:SetHorizontalAlignment(alignment)
	self.HorizontalAlignment = alignment;
end

function Image:Layout()
	if self.ImageMode == ImageMode.KeepImage then
		if self.HorizontalAlignment == HorizontalAlignment.Left then
			self.Background:SetLeft(0);
		end

		if self.HorizontalAlignment == HorizontalAlignment.Center then
			self.Background:SetLeft((self:GetWidth() - self.Background:GetWidth()) / 2);
		end
		
		if self.HorizontalAlignment == HorizontalAlignment.Right then
			self.Background:SetLeft(self:GetWidth() - self.Background:GetWidth());
		end
		
		if self.VerticalAlignment == VerticalAlignment.Top then
			self.Background:SetTop(0);
		end
		
		if self.VerticalAlignment == VerticalAlignment.Center then
			self.Background:SetTop((self:GetHeight() - self.Background:GetHeight()) / 2);
		end

		if self.VerticalAlignment == VerticalAlignment.Bottom then
			self.Background:SetTop(self:GetHeight() - self.Background:GetHeight());
		end

		self.Content:SetSize(self:GetSize());
		self.Content:PerformLayout();
		self:SetSize(self.Content:GetSize());
		--self:PerformLayout();
		return;
	end

	if self.ImageMode == ImageMode.RepeatImage then
		self.Content:SetSize(self:GetSize());
		self.Content:PerformLayout();
		self:SetSize(self.Content:GetSize());

		self.Background:SetPosition(0, 0);
		self.Background:SetSize(self:GetSize());
		self.BackgroundImage:SetSize(self.Background:GetSize());
		--self:PerformLayout();
		return;
	end

	if self.ImageMode == ImageMode.StretchImage then
		-- no content! sorry
		self.Background:SetPosition(0, 0);
		self.Background:SetStretchMode(1);
		self.Background:SetSize(self:GetSize());
		--self:PerformLayout();
		return;
	end

	if self.ImageMode == ImageMode.FitToImage then
		self.Background:SetPosition(0, 0);
		self:SetSize(self.Background:GetSize());
		self.Content:SetSize(self:GetSize());
		self.Content:PerformLayout();
		--self:PerformLayout();
		return;
	end
end

