
_G.LayoutBase = class(Turbine.UI.Control);

function LayoutBase:Constructor()
	Turbine.UI.Control.Constructor(self);
	self._layoutCalls = 0;
	self._performLayout = true;
end

function LayoutBase:SetSize(width, height)
	if self:GetWidth() == width and self:GetHeight() == height then
		-- not changed
		return;
	end

	Turbine.UI.Control.SetSize(self, width, height);
	self._performLayout = true;
end

function LayoutBase:SetWidth(width)
	if self:GetWidth() == width then
		-- not changed
		return;
	end

	Turbine.UI.Control.SetWidth(self, width);
	self._performLayout = true;
end

function LayoutBase:SetHeight(height)
	if self:GetHeight() == height then
		-- not changed
		return;
	end

	Turbine.UI.Control.SetHeight(self, height);
	self._performLayout = true;
end

function LayoutBase:SizeChanged(args)
	self._performLayout = true;
end

function LayoutBase:PerformLayout()
	if self._layoutCalls > 10 then
		Turbine.Shell.WriteLine("Warning: too many nested layout calls detected");
		return;
	end

	if not self._performLayout then
		return;
	end

	self._performLayout = false;

	self._layoutCalls = self._layoutCalls + 1;
	self:Layout();
	self._layoutCalls = self._layoutCalls - 1;
end

function LayoutBase:Invalidate()
	if type(self.ChildControls) == "table" then
		for _, child in pairs(self.ChildControls) do
			if type(child.Invalidate) == "function" then
				child:Invalidate();
			end
		end
	end

	if type(self.Content) == "table" then
		if type(self.Content.Invalidate) == "function" then
			self.Content:Invalidate();
		end
	end

	self._performLayout = true;
	self:PerformLayout();
end

function LayoutBase:Clear()
	if type(self.ChildControls) == "table" then
		for k, child in pairs(self.ChildControls) do
			if type(child.Destroy) == "function" then
				child:Destroy();
			else
				child:SetParent(nil);
			end

			for k, _ in pairs(child) do
				child[k] = nil;
			end

			self.ChildControls[k] = nil;
		end
	end

	if type(self.Content) == "table" then
		if type(self.Content.Clear) == "function" then
			self.Content:Clear();
		end
	end
end

function LayoutBase:Destroy()
	self:SetParent(nil);
	self:Clear();

	if type(self.ChildControls) == "table" then
		self.ChildControls = nil;
	end

	if type(self.Content) == "table" then
		if type(self.Content.Destroy) == "function" then
			self.Content:Destroy();
		else
			self.Content:SetParent(nil);
		end

		for k, _ in pairs(self.Content) do
			self.Content[k] = nil;
		end

		self.Content = nil;
	end

	for k, _ in pairs(self) do
		self[k] = nil;
	end
end
