
_G.ScrollViewer = class(LayoutBase);

function ScrollViewer:Constructor()
	LayoutBase.Constructor(self);

	-- content
	local control = ContentControl();
	control:SetParent(self);
	self.Content = control;

	-- listbox (internal)
	local control = Turbine.UI.ListBox();
	control:SetParent(self);
	control:SetPosition(0, 0);

	self.InternalListBox = control;

	-- vertical scrollbar
	local control = Turbine.UI.Lotro.ScrollBar();
	control:SetOrientation(Turbine.UI.Orientation.Vertical);
	control:SetWidth(10);
	control:SetParent(self);

	self.VerticalScrollbar = control;

	-- horizontal scrollbar
	local control = Turbine.UI.Lotro.ScrollBar();
	control:SetOrientation(Turbine.UI.Orientation.Horizontal);
	control:SetHeight(10);
	control:SetParent(self);

	self.HorizontalScrollbar = control;

	-- listbox (setup)
	-- lose the parent ownership!
	self.InternalListBox:AddItem(self.Content);
	self.InternalListBox:SetVerticalScrollBar(self.VerticalScrollbar);
	self.InternalListBox:SetHorizontalScrollBar(self.HorizontalScrollbar);
end

function ScrollViewer:SetContent(control)
	self.Content:SetContent(control);
	return self.Content;
end

function ScrollViewer:Destroy()
	self:SetParent(nil);
	self.InternalListBox:ClearItems();
	self.InternalListBox = nil;
	self.VerticalScrollbar = nil;
	self.HorizontalScrollbar = nil;
	LayoutBase.Destroy(self);
end

function ScrollViewer:Layout()
	self.InternalListBox:SetSize(self:GetSize());
	self.Content:SetSize(self:GetSize());
	self.Content:PerformLayout();

	if self:HasVerticalScrollbar() then
		self.VerticalScrollbar:SetPosition(self:GetWidth() - self.VerticalScrollbar:GetWidth(), 0);

		local height = self:GetHeight();
		
		if self:HasHorizontalScrollbar() and self:IsHorizontalOverflow() then
			height = height - self.HorizontalScrollbar:GetHeight();
		end

		self.VerticalScrollbar:SetHeight(height);
	end

	if self:HasHorizontalScrollbar() then
		self.HorizontalScrollbar:SetPosition(0, self:GetHeight() - self.HorizontalScrollbar:GetHeight());
		
		local width = self:GetWidth();
		
		if self:HasVerticalScrollbar() and self:IsVerticalOverflow() then
			width = width - self.VerticalScrollbar:GetWidth();
		end

		self.HorizontalScrollbar:SetWidth(width);
	end
end

function ScrollViewer:HasVerticalScrollbar()
	return self.VerticalScrollbar ~= nil and self.VerticalScrollbar:GetParent() == self;
end

function ScrollViewer:HasHorizontalScrollbar()
	return self.HorizontalScrollbar ~= nil and self.HorizontalScrollbar:GetParent() == self;
end

function ScrollViewer:IsVerticalOverflow()
	return self.Content:GetHeight() > self:GetHeight();
end

function ScrollViewer:IsHorizontalOverflow()
	return self.Content:GetWidth() > self:GetWidth();
end
