
_G.TextBlock = class(LayoutBase);

_G.TextBlockStyle = 
{
	["Font"] = Turbine.UI.Lotro.Font.Verdana14,
	["FontStyle"] = Turbine.UI.FontStyle.None,
	["ForeColor"] = Turbine.UI.Color(0.7, 0.7, 0.5),
	["OutlineColor"] = Turbine.UI.Color(0, 0, 0),
};

function TextBlock:Constructor()
	LayoutBase.Constructor(self);

	-- content
	local content = TemplateLoader.Setup(self, self:GetTemplate());
	content:SetParent(self);
	self.Content = content;

	-- self
	self.Label:SetVerticalScrollBar(self.VerticalScrollbar);
	self.Label:SetSelectable(true);
	self._autoSize    = true;
	self._markupText  = nil;
	self._plainText   = nil;
	self._numLines    = 0;
	self:SetSize(content:GetSize());
end

function TextBlock:SetAutoSize(enable)
	self._autoSize = (enable == true);
end

function TextBlock:SetText(text)
	-- workaround, keep the scrollbar on top
	self.Label:SetText(" ");
	self.Label:SelectAll();
	self.Label:SetSelectedText(text, true);
	self.Label:SetSelection(0, 0);

	self._markupText = text;
	self._plainText  = string.gsub(text, "<[^>]+>", "");
	self._numLines   = string.len(string.gsub(text, "[^\n]", "")) + 1;
end

function TextBlock:Layout()
	self.Panel:SetSize(self:GetSize());

	-- try to grow, if needed
	if self._autoSize then
		local fontData = Fonts.GetFontData(self.Label:GetFont());

		-- get a height from the text renderer
		local text;

		if self.Label:IsMarkupEnabled() then
			text = self._plainText;
		else
			text = self._markupText;
		end

		-- we need the requested width for the label, so we have to perform the layout first (see template)
		self.Panel:PerformLayout();

		local estimatedHeight = TextRenderer.MeasureTextHeight(text, fontData, self.Label:GetWidth());
		local padding = self.Label:GetParent().ContentStyle.Padding;

		-- update heights
		self.Label:SetHeight(estimatedHeight);
		self.Panel:SetHeight(estimatedHeight + padding.Top + padding.Bottom);
	end

	self.Panel:PerformLayout();
	self:SetSize(self.Panel:GetSize());
	--self:PerformLayout();
end

function TextBlock:SetTextPadding(value)
	self.Label:GetParent().ContentStyle:SetPadding(value, value, value, value);
end

function TextBlock:SetTextAlignment(value)
	self.Label:SetTextAlignment(value);
end

function TextBlock:SetMarkupEnabled(value)
	self.Label:SetMarkupEnabled(value);
end

function TextBlock:SetFontStyle(value)
	self.Label:SetFontStyle(value);
end

function TextBlock:SetForeColor(value)
	self.Label:SetForeColor(value);
end

function TextBlock:SetFont(value)
	if not Fonts.GetFontData(value) then
		-- we don't have any information about the font ... use the default font
		value = Fonts.DefaultLotroFont;
	end

	self.Label:SetFont(value);
end

function TextBlock:SetOutlineColor(value)
	self.Label:SetOutlineColor(value);
end

function TextBlock:SetScrollbarColor(color)
	-- the ContentControl
	self.VerticalScrollbar:GetParent():SetBackColor(color);
end

function TextBlock:GetTemplate(bindings)
	return {
		["Type"] = StackPanel,
		["Name"] = "Panel",
		["Width"] = 30,
		["Height"] = 20,
		["IsVertical"] = false,
		["Content"] =
		{
			{
				["Type"] = Turbine.UI.Label,
				["Name"] = "Label",
				["Width"] = 20,
				["Height"] = 20,
				["TextAlignment"] = Turbine.UI.ContentAlignment.TopLeft,
				["MarkupEnabled"] = true,
				["Font"] = TextBlockStyle.Font,
				["FontStyle"] = TextBlockStyle.FontStyle,
				["ForeColor"] = TextBlockStyle.ForeColor,
				["OutlineColor"] = TextBlockStyle.OutlineColor,
				["Multiline"] = true,
				["Content.DynamicWidth"] = 1;
			},
			{
				["Type"] = Turbine.UI.Lotro.ScrollBar,
				["Name"] = "VerticalScrollbar",
				["Width"] = 10,
				["Height"] = 10,
				["Orientation"] = Turbine.UI.Orientation.Vertical,
			},
		},
	};
end
