
TextRenderer = {};

-- naive
function TextRenderer.MeasureTextHeight(text, fontData, width)
	local ratio = fontData.Ratio or {};
	local fontHeight = fontData.FontHeight;
	local w = 0;
	local h = fontHeight;

	if text == nil then
		return 0;
	end

	if not (width > 0) then
		return 0;
	end

	for word in string.gmatch(text, "[^- \n]*[- \n]?") do
		local start = w;

		for c in utf8.chars(word, true) do
			if c == "\n" then
				w = 0;
				h = h + fontHeight;
			else
				local charWidth = ratio[c] or fontHeight;

				w = w + charWidth;

				if not (w < width) then
					if start == 0 then
						if w > 0 or h > fontHeight then
							h = h + fontHeight;
						end

						w = charWidth;

						if c == " " then
							w = 0;
						end

						start = w;
					else
						w = w - start;
						h = h + fontHeight;
					end
				end
			end
		end
	end

	return h;
end

-- naive
function TextRenderer.MeasureText(text, fontData)
	local ratio = fontData.Ratio;
	local fontHeight = fontData.FontHeight;
	local w = 0;
	local h = fontHeight;
	local maxwidth = 0;

	if text == nil then
		return {["Width"] = 0, ["Height"] = 0};
	end

	for c in utf8.chars(text, true) do
		if c == "\n" then
			maxwidth = math.max(maxwidth, w + 1);
			w = 0;
			h = h + fontHeight;
		else
			local charWidth = ratio[c] or fontHeight;
			w = w + charWidth;
		end
	end

	maxwidth = math.max(maxwidth, w);

	return {["Width"] = maxwidth, ["Height"] = h};
end
