
GoldsellerPolicyWindow = class(BaseWindow);

function GoldsellerPolicyWindow:Constructor(filterEngine)
	BaseWindow.Constructor(self);
	self.Engine = filterEngine;
	self.Policy = self.Engine:GetPolicy("GoldsellerPolicy");

	local bindings = {
		["HeuristicWords"] = table.concat(self.Policy.Data.SpamHeuristic.Words, " "),
		["HeuristicSymbols"] = table.concat(self.Policy.Data.SpamHeuristic.Symbols, " "),

		["ThresholdValue"] = self.Policy.Data.SpamHeuristic.Threshold,
		["ThresholdScrollBarValueChanged"] = function(sender, args)
			local threshold = math.floor(sender:GetValue() / sender:GetSmallChange());

			if self.ThresholdTextBlock then
				self.ThresholdTextBlock:SetText(tostring(threshold));
			end
		end;

		["ApplyButtonClick"] = function(sender, args)
			self.Policy:SetHeuristicTables(self.HeuristicWordsTextBlock:GetText(), self.HeuristicSymbolsTextBlock:GetText());

			local threshold = math.floor(self.ThresholdScrollBar:GetValue() / self.ThresholdScrollBar:GetSmallChange());
			self.Policy:SetThreshold(threshold);
		end,
	};

	local content = TemplateLoader.Setup(self, self:GetTemplate(bindings));
	self:SetContent(content);
	self:RegisterCallbacks();
	self:SetText("Goldseller Policy");
	self:FitToContent();
	self:SetMaximumSize(1000, 790);
end

function GoldsellerPolicyWindow:RegisterCallbacks()
	self._registrations = self._registrations or {};
	
	-- helper
	local function add(object, event, callback)
		Events.AddCallback(object, event, callback);
		table.insert(self._registrations, {["object"] = object, ["event"] = event, ["callback"] = callback});
	end

	add(self.Policy, "HeuristicTablesChanged", function(sender, args)
		self.HeuristicWordsTextBlock:SetText(table.concat(self.Policy.Data.SpamHeuristic.Words, " "));
		self.HeuristicSymbolsTextBlock:SetText(table.concat(self.Policy.Data.SpamHeuristic.Symbols, " "));
	end);

	add(self.Policy, "ThresholdChanged", function(sender, args)
		self.ThresholdScrollBar:SetValue(self.Policy.Data.SpamHeuristic.Threshold * self.ThresholdScrollBar:GetSmallChange());
	end);
end

function GoldsellerPolicyWindow:UnregisterCallbacks()
	if self._registrations then
		for _, reg in pairs(self._registrations) do
			Events.RemoveCallback(reg.object, reg.event, reg.callback);
		end

		self._registrations = nil;
	end
end

function GoldsellerPolicyWindow:Closing(args)
	self:UnregisterCallbacks();
	self.Engine = nil;
	self.Policy = nil;
	BaseWindow.Closing(self, args);
end

function GoldsellerPolicyWindow:GetTemplate(bindings)
	return {
		["Type"] = StackPanel,
		["Width"] = 500,
		["Height"] = 300,
		["FitToContent"] = false,
		["Content"] =
		{
			{
				["Type"] = Border,
				["BorderWidth"] = 1,
				["BorderColor"] = Turbine.UI.Color(0.98, 0.2, 0.1, 0.2),
				["Content.DynamicHeight"] = 1,
				["Content"] = 
				{
					{
						["Type"] = Image,
						["ImageMode"] = ImageMode.KeepImage,
						["VerticalAlignment"] = VerticalAlignment.Bottom,
						["HorizontalAlignment"] = HorizontalAlignment.Right,
						["Background"] = Images.Goldseller,
						["BlendMode"] = Turbine.UI.BlendMode.Multiply,
						["ImageBackColor"] = Turbine.UI.Color(0.8, 0.7, 0.8),
						["BackColor"] = Turbine.UI.Color(0.98, 0.3, 0.2, 0.3),
						["Content.DynamicWidth"] = 1,
						["Content.DynamicHeight"] = 1,
						["Content"] = 
						{
							{
								["Type"] = ScrollViewer,
								--["BackColor"] = Turbine.UI.Color(0.98, 0.2, 0.1, 0.2),
								["Content"] = 
								{
									{
										["Type"] = StackPanel,
										["IsVertical"] = false,
										["Content"]=
										{
											{
												["Type"] = StackPanel,
												["IsVertical"] = false,
												--["Content.BackColor"] = Turbine.UI.Color(0.98, 0.3, 0.2, 0.3),
												["Content.DynamicWidth"] = 8,
												["Content"]=
												{
													{
														["Type"] = StackPanel,
														["FitToContent"] = false,
														["Content.DynamicWidth"] = 1,
														["Content.Padding"] = {Left = 4, Right = 4, Bottom = 4},
														["Content"] =
														{
															{
																["Type"] = TextBlock,
																["Text"] = "Settings",
																["TextAlignment"] = Turbine.UI.ContentAlignment.MiddleCenter,
																["Content.VerticalAlignment"] = VerticalAlignment.Center,
																["ForeColor"] = Turbine.UI.Color(0.98, 0.7, 0.6, 0.3),
																["Content.MinHeight"] = 50,
																["Content.DynamicWidth"] = 1,
															},
															{
																["Type"] = StackPanel,
																["IsVertical"] = false,
																["Content.Padding"] = {Top = 4},
																["Content"] =
																{
																	{
																		["Type"] = TextBlock,
																		["Text"] = "Words",
																		["Width"] = 100,
																	},
																	{
																		["Type"] = Turbine.UI.TextBox,
																		["Name"] = "HeuristicWordsTextBlock",
																		["Text"] = bindings["HeuristicWords"],
																		["Multiline"] = false,
																		["Selectable"] = true,
																		["Font"] = TextBlockStyle.Font,
																		--["ForeColor"] = TextBlockStyle.ForeColor,
																		["ForeColor"] = Turbine.UI.Color(0.98, 0.2, 0.1, 0.2),
																		["ForeStyle"] = TextBlockStyle.ForeStyle,
																		["BackColor"] = Turbine.UI.Color(0.98, 0.5, 0.4, 0.5),
																		["TextAlignment"] = Turbine.UI.ContentAlignment.MiddleLeft,
																		["Height"] = 20,
																		["Content.DynamicWidth"] = 1,
																	},
																},
															},
															{
																["Type"] = StackPanel,
																["IsVertical"] = false,
																["Content.Padding"] = {Top = 4},
																["Content"] =
																{
																	{
																		["Type"] = TextBlock,
																		["Text"] = "Symbols",
																		["Width"] = 100,
																	},
																	{
																		["Type"] = Turbine.UI.TextBox,
																		["Name"] = "HeuristicSymbolsTextBlock",
																		["Text"] = bindings["HeuristicSymbols"],
																		["Multiline"] = false,
																		["Selectable"] = true,
																		["Font"] = TextBlockStyle.Font,
																		--["ForeColor"] = TextBlockStyle.ForeColor,
																		["ForeColor"] = Turbine.UI.Color(0.98, 0.2, 0.1, 0.2),
																		["ForeStyle"] = TextBlockStyle.ForeStyle,
																		["BackColor"] = Turbine.UI.Color(0.98, 0.5, 0.4, 0.5),
																		["TextAlignment"] = Turbine.UI.ContentAlignment.MiddleLeft,
																		["Height"] = 20,
																		["Content.DynamicWidth"] = 1,
																	},
																},
															},
															{
																["Type"] = StackPanel,
																["IsVertical"] = false,
																["Content.Padding"] = {Top = 4},
																["Content"] =
																{
																	{
																		["Type"] = TextBlock,
																		["Text"] = "Threshold",
																		["Width"] = 100,
																	},
																	{
																		["Type"] = Border,
																		["BorderWidth"] = 1,
																		["BorderColor"] = Turbine.UI.Color(0.98, 0.5, 0.4, 0.5),
																		["BackColor"] = Turbine.UI.Color(0.98, 0.3, 0.2, 0.3),
																		["Content.VerticalAlignment"] = VerticalAlignment.Center,
																		["Content.DynamicWidth"] = 1,
																		["Content"] =
																		{
																			{
																				["Type"] = Turbine.UI.Lotro.ScrollBar,
																				["Name"] = "ThresholdScrollBar",
																				["Orientation"] = Turbine.UI.Orientation.Horizontal,
																				["Height"] = 10,
																				["Width"] = 100,
																				["Minimum"] = 20,
																				["Maximum"] = 100,
																				["LargeChange"] = 10,
																				["SmallChange"] = 10,
																				["Value"] = bindings["ThresholdValue"] * 10,
																				["ValueChanged"] = bindings["ThresholdScrollBarValueChanged"],
																				["Content.VerticalAlignment"] = VerticalAlignment.Center,
																				["Content.Padding"] = 4,
																			},
																		},
																	},
																	{
																		["Type"] = TextBlock,
																		["Name"] = "ThresholdTextBlock",
																		["Text"] = bindings["ThresholdValue"],
																		["TextAlignment"] = Turbine.UI.ContentAlignment.MiddleLeft,
																		["Content.PaddingLeft"] = 10,
																	},
																},
															},
														},
													},
												}
											},
											{
												["Type"] = Image,
												["ImageMode"] = ImageMode.KeepImage,
												["VerticalAlignment"] = VerticalAlignment.Bottom,
												["HorizontalAlignment"] = HorizontalAlignment.Right,
												["Background"] = Images.Goldseller,
												["BlendMode"] = Turbine.UI.BlendMode.Multiply,
												["BackColor"] = Turbine.UI.Color(0.98, 0.5, 0.3, 0.5),
												["Content.DynamicWidth"] = 5,
												["Content.DynamicHeight"] = 1,
												["Content"] =
												{
													{
														["Type"] = StackPanel,
														--["Content.BackColor"] = Turbine.UI.Color(0.98, 0.4, 0.2, 0.3),
														["Content.DynamicWidth"] = 5,
														["Content"] =
														{
															{
																["Type"] = TextBlock,
																["Text"] = "Description",
																["TextAlignment"] = Turbine.UI.ContentAlignment.MiddleCenter,
																["Content.VerticalAlignment"] = VerticalAlignment.Center,
																["ForeColor"] = Turbine.UI.Color(0.98, 0.7, 0.6, 0.3),
																["Content.MinHeight"] = 50,
																["Content.DynamicWidth"] = 1,
															},
															{
																["Type"] = TextBlock,
																["Text"] = Strings.GoldsellerPolicyDescription,
																["AutoSize"] = false,
																["TextPadding"] = 4,
																--["Content.BackColor"] = Turbine.UI.Color(0.98, 0.5, 0.3, 0.4),
																["Content.DynamicHeight"] = 1,
															},
														},
													},
												},
											},
										},
									},
								},
							},
						},
					},
				},
			},
			{
				["Type"] = StackPanel,
				["IsVertical"] = false,
				["Content.PaddingTop"] = 10,
				["Content.DynamicWidth"] = 1,
				["Content"] = 
				{
					{
						["Type"] = Button,
						["Text"] = "Apply",
						["Content.MaxWidth"] = 150,
						["Content.PaddingTop"] = 10,
						["Content.PaddingRight"] = 4,
						["Content.PaddingBottom"] = 4,
						["Content.HorizontalAlignment"] = HorizontalAlignment.Center,
						["Content.DynamicWidth"] = 1,
						["Click"] = bindings["ApplyButtonClick"],
					},
				},
			},
		},
	};
end
