
SpamPolicyWindow = class(BaseWindow);

function SpamPolicyWindow:Constructor(filterEngine)
	BaseWindow.Constructor(self);
	self.Engine = filterEngine;
	self.Policy = self.Engine:GetPolicy("SpamPolicy");
	self._autoRefresh = true;

	local bindings = {
		["ResetNoSpamButtonClick"] = function(sender, args)
			self.Policy:ResetNoSpam();
		end,

		["ResetIsSpamButtonClick"] = function(sender, args)
			self.Policy:ResetIsSpam();
		end,
		
		["AutoRefreshCheckBoxName"] = "AutoRefreshCheckBox",
		["AutoRefreshChecked"] = self._autoRefresh,
		["AutoRefreshCheckedChanged"] = function(sender, args)
			self._autoRefresh = sender:IsChecked();
			self:UpdateStats();
		end,
	};

	local content = TemplateLoader.Setup(self, self:GetTemplate(bindings));
	self:SetContent(content);
	self:RegisterCallbacks();
	self:SetText("Spam Policy");
	self:FitToContent();
	self:UpdateStats();
end

function SpamPolicyWindow:RegisterCallbacks()
	self._registrations = self._registrations or {};
	
	-- helper
	local function add(object, event, callback)
		Events.AddCallback(object, event, callback);
		table.insert(self._registrations, {["object"] = object, ["event"] = event, ["callback"] = callback});
	end

	add(self.Engine, "MessageCompleted", function(sender, args)
		self:UpdateStats(args);
	end);

	add(self.Policy, "Reseted", function(sender, args)
		self:UpdateStats(args);
	end);
end

function SpamPolicyWindow:UnregisterCallbacks()
	if self._registrations then
		for _, reg in pairs(self._registrations) do
			Events.RemoveCallback(reg.object, reg.event, reg.callback);
		end

		self._registrations = nil;
	end
end

function SpamPolicyWindow:Closing(args)
	self:UnregisterCallbacks();
	self.Engine = nil;
	self.Policy = nil;
	BaseWindow.Closing(self, args);
end

function SpamPolicyWindow:UpdateStats(args)
	if self._autoRefresh == false then
		return;
	end

	local buffNoSpam = {};
	local buffIsSpam = {};

	for k, v in pairs(self.Policy._noSpamWords) do
		table.insert(buffNoSpam, k .. " " .. v);
	end

	for k, v in pairs(self.Policy._isSpamWords) do
		table.insert(buffIsSpam, k .. " " .. v);
	end

	table.sort(buffNoSpam);
	self.NoSpamTextBlock:SetText(table.concat(buffNoSpam, "\n"));

	table.sort(buffIsSpam);
	self.IsSpamTextBlock:SetText(table.concat(buffIsSpam, "\n"));
end

function SpamPolicyWindow:GetTemplate(bindings)
	return {
		["Type"] = StackPanel,
		["Width"] = 400,
		["Height"] = 300,
		["FitToContent"] = false,
		["Content"] =
		{
			{
				["Type"] = StackPanel,
				["IsVertical"] = false,
				["Content.DynamicHeight"] = 1,
				["Content"] = 
				{
					{
						["Type"] = StackPanel,
						["Content.DynamicWidth"] = 1,
						["Content"] =
						{
							{
								["Type"] = TextBlock,
								["Text"] = "No Spam",
							},
							{
								["Type"] = Border,
								["BorderWidth"] = {Left = 10, Top = 10, Bottom = 10},
								["BorderColor"] = Turbine.UI.Color(0.98, 0.1, 0.2, 0.1),
								["Content.DynamicHeight"] = 1,
								["Content"] = 
								{
									{
										["Type"] = TextBlock,
										["Name"] = "NoSpamTextBlock",
										["AutoSize"] = false;
										["TextPadding"] = 4,
										["ScrollbarColor"] = Turbine.UI.Color(0.98, 0.1, 0.2, 0.1),
										["Content.BackColor"] = Turbine.UI.Color(0.98, 0.2, 0.3, 0.2),
										["Content.DynamicHeight"] = 1,
									},
								},
							},
						    {
								["Type"] = Button,
								["Text"] = "Reset",
								["Content.PaddingTop"] = 10,
								["Content.DynamicWidth"] = 1,
								["Click"] = bindings["ResetNoSpamButtonClick"],
							},
						},
					},
					{
						["Type"] = Border,
						["Width"] = 10,
					},
					{
						["Type"] = StackPanel,
						["Content.DynamicWidth"] = 1,
						["Content"] =
						{
							{
								["Type"] = TextBlock,
								["Text"] = "Spam",
							},
							{
								["Type"] = Border,
								["BorderWidth"] = {Left = 10, Top = 10, Bottom = 10},
								["BorderColor"] = Turbine.UI.Color(0.98, 0.2, 0.1, 0.1),
								["Content.DynamicHeight"] = 1,
								["Content"] = 
								{
									{
										["Type"] = TextBlock,
										["Name"] = "IsSpamTextBlock",
										["AutoSize"] = false;
										["TextPadding"] = 4,
										["ScrollbarColor"] = Turbine.UI.Color(0.98, 0.2, 0.1, 0.1),
										["Content.BackColor"] = Turbine.UI.Color(0.98, 0.3, 0.2, 0.2),
										["Content.DynamicHeight"] = 1,
									},
								},
							},
						    {
								["Type"] = Button,
								["Text"] = "Reset",
								["Content.PaddingTop"] = 10,
								["Content.DynamicWidth"] = 1,
								["Click"] = bindings["ResetIsSpamButtonClick"],
							},
						},
					},
				},
			},
			{
				["Type"] = Border,
				["BorderWidth"] = {["Top"] = 1},
				["Content.PaddingTop"] = 10,
				["Content.PaddingBottom"] = 10,
			},
			{
				["Type"] = CheckBox,
				["Name"] = bindings["AutoRefreshCheckBoxName"],
				["Text"] = "Auto Refresh",
				["Checked"] = bindings["AutoRefreshChecked"],
				["CheckedChanged"] = bindings["AutoRefreshCheckedChanged"],
			},
		},
	};
end
